﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace TChart3DAnimation
{
    public partial class Form1 : Form
    {
        int dataCnt = 1000;

        private bool autoRepaint;   //true: chart repaints itself, false: thread forces chart repaint 

        private bool stopAnimationThread = false;
        private Thread animationThread = null;
        private object chartSynchro;    //synchronizes thread and chart painting when autoRepaint=true

        //data buffers
        double[] xBuffer1;
        double[] yBuffer1;
        double[] zBuffer1;

        private Random random;

        public Form1()
        {
            InitializeComponent();

            this.autoRepaint = true;
            this.chartSynchro = new object();

            this.random = new Random();

            #region initialize data
            this.xBuffer1 = new double[dataCnt];
            this.yBuffer1 = new double[dataCnt];
            this.zBuffer1 = new double[dataCnt];

            this.xBuffer1[0] = 0;
            this.yBuffer1[0] = dataCnt;
            this.zBuffer1[0] = 0;

            int tmpCnt = this.dataCnt / 4;
            for (int i = 1; i < this.dataCnt; i++)
            {
                if (i < tmpCnt)
                {
                    this.xBuffer1[i] = 0;
                    this.yBuffer1[i] = i;
                    this.zBuffer1[i] = 0;
                }
                else if (i < 2 * tmpCnt)
                {
                    this.xBuffer1[i] = i - tmpCnt;
                    this.yBuffer1[i] = this.yBuffer1[tmpCnt-1];
                    this.zBuffer1[i] = (i-tmpCnt)/3;
                }
                else if (i < 3 * tmpCnt)
                {
                    this.xBuffer1[i] = this.xBuffer1[2 * tmpCnt - 1];
                    this.yBuffer1[i] = this.yBuffer1[tmpCnt - 1] + 2 * tmpCnt - i;
                    this.zBuffer1[i] = this.zBuffer1[2 * tmpCnt - 1] + (i - 2*tmpCnt) / 5;
                }
                else
                {
                    this.xBuffer1[i] = this.xBuffer1[2 * tmpCnt - 1] + 3 * tmpCnt-i;
                    this.yBuffer1[i] = 0;
                    this.zBuffer1[i] = this.zBuffer1[3 * tmpCnt - 1] - (i - 3 * tmpCnt);
                }
            }

            this.points3D1.Add(this.xBuffer1, this.yBuffer1, this.zBuffer1);
            #endregion

            this.checkBoxAutoUpdate.Checked = this.autoRepaint;
            this.tChart1.AutoRepaint = this.autoRepaint;
            this.timer1.Enabled = false;// this.autoRepaint;
            this.buttonKickTheChart.Enabled = this.autoRepaint;

            this.animationThread = new Thread(new ParameterizedThreadStart(AnimationThreadProc));
            this.stopAnimationThread = false;
            this.animationThread.Start(this);
        }

        public static void AnimationThreadProc(object data)
        {
            Form1 _this = data as Form1;
            if (_this != null)
            {
                int n = 0;
                double theta = 0;
                double deltaTheta = 2 * Math.PI / _this.dataCnt;

                DateTime startTime = DateTime.Now;
                while (!_this.stopAnimationThread)
                {
                    if (_this.autoRepaint)
                        Monitor.Enter(_this.chartSynchro);

                    _this.points3D1.BeginUpdate();
                    #region data calculation
                    Steema.TeeChart.Styles.ValueList xValues1 = _this.points3D1.XValues;
                    Steema.TeeChart.Styles.ValueList yValues1 = _this.points3D1.YValues;
                    Steema.TeeChart.Styles.ValueList zValues1 = _this.points3D1.ZValues;

                    xValues1[0] = _this.xBuffer1[0] + 100 * Math.Cos(theta);
                    yValues1[0] = _this.yBuffer1[0] + 100 * Math.Sin(theta);

                    for (int i = 1; i < _this.dataCnt; i++)
                    {
                        xValues1[i] = _this.xBuffer1[i] + _this.xBuffer1[i] / 5 * _this.random.NextDouble();
                        yValues1[i] = _this.yBuffer1[i] + _this.yBuffer1[i] / 5 * _this.random.NextDouble();
                        zValues1[i] = _this.zBuffer1[i] + _this.zBuffer1[i] / 5 * _this.random.NextDouble();
                    }
                    #endregion
                    _this.points3D1.EndUpdate();
                    if (!_this.autoRepaint)
                        _this.ThreadChartInvoker(_this.tChart1, eTCI.e_TCI_Refresh, null);
                   
                    System.Threading.Thread.Sleep(5);

                    #region calculate and update FPS
                    if (n == 9)
                    {
                        DateTime endTime = DateTime.Now;
                        TimeSpan timeSpan = endTime - startTime;
                        startTime = endTime;
                        double totalSeconds = timeSpan.TotalSeconds;
                        double fps = 10.0 / totalSeconds;
                        _this.ThreadChartInvoker(null, eTCI.e_TCI_UpdateRealCycleTime, new object[] { fps });
                        n = 0;
                    }
                    else
                        n++;
                    #endregion

                    if (theta >= 2 * Math.PI)
                    {
                        theta = theta % 2 * Math.PI;
                        Console.Beep();
                    }
                    else
                        theta += deltaTheta;
                }
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.stopAnimationThread = true;
            if (this.animationThread != null && !this.animationThread.Join(5000))
            {
                try
                {
                    this.animationThread.Abort();
                }
                catch (ThreadAbortException ex)
                { }
            }
            this.animationThread = null;
        }

        private enum eTCI { e_TCI_Refresh, e_TCI_UpdateRealCycleTime };
        private delegate void ThreadChartInvokerDelegate(Steema.TeeChart.TChart chart, eTCI method, object[] parameters);
        private void ThreadChartInvoker(Steema.TeeChart.TChart chart, eTCI method, object[] parameters)
        {
            if (InvokeRequired)
            {
                Invoke(new ThreadChartInvokerDelegate(ThreadChartInvoker), new object[] { chart, method, parameters });
            }
            else
            {
                switch (method)
                {
                    case eTCI.e_TCI_Refresh:
                        //this.points3D1.Repaint();
                        chart.Invalidate();
                        //this.points3D1.RefreshSeries();
                        //chart.Refresh();
                        break;
                    case eTCI.e_TCI_UpdateRealCycleTime:
                        this.labelControlFPS.Text = string.Format("FPS: {0:f2}", (double)parameters[0]);
                        break;
                }
            }
        }

        private void tChart1_AfterDraw(object sender, Steema.TeeChart.Drawing.Graphics3D g)
        {
            try
            {
                Monitor.Exit(this.chartSynchro);
            }
            catch (SynchronizationLockException ex)
            { }
        }

        //timer is inative
        private void timer1_Tick(object sender, EventArgs e)
        {
            this.tChart1.Invalidate();
        }

        private void checkBoxAutoUpdate_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = sender as CheckBox;
            if (checkBox != null)
            {
                this.autoRepaint = checkBox.Checked;
                this.tChart1.AutoRepaint = this.autoRepaint;
                this.buttonKickTheChart.Enabled = this.autoRepaint;
            }
        }

        private void buttonKickTheChart_Click(object sender, EventArgs e)
        {
            this.tChart1.Invalidate();
        }
    }
}
